
#include <htc.h>

#include "HW_profile.h"

void init(void){
    // IO
    nWPUEN = 0; // OPTION_REG bit7, weak pull up enable
    //OPTION_REG = 0b00110010;    // bit 7 WPUEN: Weak Pull-up Enable bit
                                //    0 = Weak pull-ups are enabled by individual WPUx latch values
                                // bit 6 INTEDG: Interrupt Edge Select bit
                                //    0 = Interrupt on falling edge of INT pin
                                // bit 5 TMR0CS: Timer0 Clock Source Select bit
                                //    0 = Internal instruction cycle clock (FOSC/4)
                                // bit 4 TMR0SE: Timer0 Source Edge Select bit
                                // bit 3 PSA: Prescaler Assignment bit
                                //    0 = Prescaler is assigned to the Timer0 module
                                // bit 2-0 PS<2:0>: Prescaler Rate Select bits
                                //      010 = 1:8   // TMR0 preod = 2.048 ms

    //ANSELA = 0b00010010;    // RA0,1 = AN0,1 = analog
    //SET_ANSEL();
    ANSELA = 0b00000011;
    ANSELC = 0;
    WPUA = 0b11011000;
    WPUC = 0xFF;
    LATA = 0;
    TRISA = 0b11111010;
    LATC = 0;
    TRISC = TRIS_LED;
    LED_POW_OFF();
    SPK_OFF();

    IOCF_SW1 = 0;
    IOC_SW1 = 1;
    INTCON = 0b01001000;    // PIE(bit6) = 1; IOCIE(bit3) = 1; IOCIF(bit0) = 0;

    //comparetor
    CM1CON0 = 0b10100100;
    //bit 7 CxON: Comparator Enable bit
    //   1 = Comparator is enabled and consumes no active power
    //bit 6 CxOUT: Comparator Output bit
    //bit 5 CxOE: Comparator Output Enable bit
    //   1 = CxOUT is present on the CxOUT pin.
    //bit 4 CxPOL: Comparator Output Polarity Select bit
    //    0 = Comparator output is not inverted
    //bit 3 Unimplemented: Read as ?0?
    //bit 2 CxSP: Comparator Speed/Power Select bit
    //   1 = Comparator operates in normal power, higher speed mode
    //bit 1 CxHYS: Comparator Hysteresis Enable bit
    //    0 = Comparator hysteresis disabled
    //bit 0 CxSYNC: Comparator Output Synchronous Mode bit
    //   0 = Comparator output to Timer1 and I/O pin is asynchronous.

    CM1CON1 = 0b00010000;
    //  bit 7 CxINTP: Comparator Interrupt on Positive Going Edge Enable bits
    //    0 = No interrupt flag will be set on a positive going edge of the CxOUT bit
    //  bit 6 CxINTN: Comparator Interrupt on Negative Going Edge Enable bits
    //    0 = No interrupt flag will be set on a negative going edge of the CxOUT bit
    //  bit 5-4 CxPCH<1:0>: Comparator Positive Input Channel Select bits
    //       00 = CxVP connects to CxIN+ pin
    //      01 = CxVP connects to DAC Voltage Reference
    //  bit 3-2 Unimplemented: Read as ?0?
    //  bit 1-0 CxNCH<1:0>: Comparator Negative Input Channel Select bits
    //  00 = CxVN connects to C12IN0- pin

    //DAC
    DACCON0 = 0b10100000;
    //bit 7
    //  DACEN: DAC Enable bit
    //      1 = DAC is enabled
    //  bit 6
    //  DACLPS: DAC Low-Power Voltage State Select bit
    //      0 = DAC Negative reference source selected
    //  bit 5
    //  DACOE: DAC Voltage Output Enable bit
    //  1 = DAC voltage level is also an output on the DACOUT pin
    //  bit 4
    //  Unimplemented: Read as ?0?
    //  bit 3-2
    //  DACPSS<1:0>: DAC Positive Source Select bits
    //      00 = VDD
    //  bit 1-0
    //  Unimplemented: Read as ?0?
    DACCON1 = 0x10;
    //  bit 7-5 Unimplemented: Read as ?0?
    //  bit 4-0 DACR<4:0>: DAC Voltage Output Select bits


    // Timer 0
    OPTION_REG &= 0b11010111;
        //bit 5 TMR0CS: Timer0 Clock Source Select bit
        //	1 = Transition on RA4/T0CKI pin
        //	0 = Internal instruction cycle clock (FOSC/4)
        //bit 4 TMR0SE: Timer0 Source Edge Select bit
        //	1 = Increment on high-to-low transition on RA4/T0CKI pin
        //	0 = Increment on low-to-high transition on RA4/T0CKI pin
        //bit 3 PSA: Prescaler Assignment bit
        //	1 = Prescaler is not assigned to the Timer0 module
        //	0 = Prescaler is assigned to the Timer0 module
        //bit 2-0 PS<2:0>: Prescaler Rate Select bits
        //	111 = 1:256

    //Timer 1
    T1CON = 0b10000101;
        //	bit 7-6 TMR1CS<1:0>: Timer1 Clock Source Select bits
        //		11 =Timer1 clock source is Capacitive Sensing Oscillator (CAPOSC)
        //              10 = Timer1 clock source is pin or oscillator
        //	bit 5-4 T1CKPS<1:0>: Timer1 Input Clock Prescale Select bits
        //		11 = 1:8 Prescale value
        //		10 = 1:4 Prescale value
        //		01 = 1:2 Prescale value
        //		00 = 1:1 Prescale value
        //	bit 3 T1OSCEN: LP Oscillator Enable Control bit
        //		0 = Dedicated Timer1 oscillator circuit disabled
        //	bit 2 T1SYNC: Timer1 External Clock Input Synchronization Control bit
        //		TMR1CS<1:0> = 1X
        //			1 = Do not synchronize external clock input
        //			0 = Synchronize external clock input with system clock (FOSC)
        //	bit 1 Unimplemented: Read as ?0?
        //	bit 0 TMR1ON: Timer1 On bit
        //		1 = Enables Timer1
        //		0 = Stops Timer1

    T1GCON = 0b11100001;
        //	bit 7 TMR1GE: Timer1 Gate Enable bit
        //		If TMR1ON = 1:
        //			1 = Timer1 counting is controlled by the Timer1 gate function
        //	bit 6 T1GPOL: Timer1 Gate Polarity bit
        //		1 = Timer1 gate is active-high (Timer1 counts when gate is high)
        //		0 = Timer1 gate is active-low (Timer1 counts when gate is low)
        //	bit 5 T1GTM: Timer1 Gate Toggle Mode bit
        //		1 = Timer1 Gate Toggle mode is enabled
        //		0 = Timer1 Gate Toggle mode is disabled and toggle flip flop is cleared
        //			Timer1 gate flip-flop toggles on every rising edge.
        //	bit 4 T1GSPM: Timer1 Gate Single-Pulse Mode bit
        //		1 = Timer1 gate Single-Pulse mode is enabled and is controlling Timer1 gate
        //		0 = Timer1 gate Single-Pulse mode is disabled
        //	bit 3 T1GGO/DONE: Timer1 Gate Single-Pulse Acquisition Status bit
        //		1 = Timer1 gate single-pulse acquisition is ready, waiting for an edge
        //		0 = Timer1 gate single-pulse acquisition has completed or has not been started
        //	bit 2 T1GVAL: Timer1 Gate Current State bit
        //		Indicates the current state of the Timer1 gate that could be provided to TMR1H:TMR1L.
        //		Unaffected by Timer1 Gate Enable (TMR1GE).
        //	bit 1-0 T1GSS<1:0>: Timer1 Gate Source Select bits
        //		01 = Timer0 overflow output

    TMR1 = 0;
    TMR1GIF = 0;
    TMR1GIE = 1;
    PEIE = 1;
//	GIE = 1;
//    TMR0CS = 0;
//    TMR1ON = 1;

    // PWM
    T2CON = 0b00000111;
        // bit 7 Unimplemented: Read as ?0?
        // bit 6-3 TxOUTPS<3:0>: Timerx Output Postscaler Select bits
        //      0000 = 1:1 Postscaler
        // bit 2 TMRxON: Timerx On bit
        //      1 = Timerx is on
        //      0 = Timerx is off
        // bit 1-0 TxCKPS<1:0>: Timer2-type Clock Prescale Select bits
        //     01 = Prescaler is 4
        //      10 = Prescaler is 16
        //      11 = Prescaler is 64
    PR2 = 0xFF;

    CCP1CON = 0b00001111;
    //CCP1CON = 0b00000000;
    //bit 7-6
    //If CCP1M<3:2> = 11:
    //  00 = Single output; P1A modulated; P1B, P1C, P1D assigned as port pins
    //bit 5-4
    //DC1B<1:0>: PWM Duty Cycle Least Significant bits
    //bit 3-0
    //CCP1M<3:0>: ECCP1 Mode Select bits
    //  PWM mode:
    //  1100 = PWM mode: P1A, P1C active-high; P1B, P1D active-high
    //  1111 = PWM mode: P1A, P1C active-low; P1B, P1D active-low

    CCPR1L = 0x00;  //Duty = 0%
//    CCPR1L = 0x7F;  //Duty = 50%



}
