#include <htc.h>
#include "HW_profile.h"

#ifndef _XTAL_FREQ
#define _XTAL_FREQ 4000000
#endif


#pragma config FOSC = INTOSC // INTOSCIO oscillator:
#pragma config MCLRE = OFF // MCLR pin function is RA3
#pragma config PWRTE = ON // PWRT enabled
#pragma config WDTE = OFF // WDT disabled
#pragma config PLLEN = OFF //
#pragma config BOREN = OFF // BOR enabled
#pragma config BORV = LO // BOR enabled
#pragma config CLKOUTEN = OFF
#pragma config LVP = OFF

#define FREQ_LOW 0x4000
#define DET_TH 0x22 // threshold of delta freq to detect
#define DET_MAX 0x1FF
#define DLT_STABLE 0x04

//unsigned char flg;
unsigned int freq;
unsigned int dlt_freq;
unsigned int freq_ref;
unsigned short long sum_freq;
unsigned int freq_avg_old;


void delay_50ms(void){
    __delay_ms(50);
}

void delay_1sec(void){
    unsigned char i;
    for(i=0; i<20; i++)
        delay_50ms();
}

void go_to_bed(void);

int main(void) {
    unsigned char i;

//    OSCCON = 0b01111000;	// SPLLEN = OFF, 16MHz, Clock determined by FOSC<2:0> in
//    OSCCON = 0b11110000;	// SPLLEN = ON, 8MHz(x4=32MHz), Clock determined by FOSC<2:0> in Configuration Word 1.
//    OSCCON = 0b01110000;    // bit7: SPLLEN = OFF
                            // bit6-3: 110 = 8MHz
                            // bit 1-0: Clock determined by Config
    OSCCON = 0b01101000;    // bit7: SPLLEN = OFF
                            // bit6-3: 101 = 4MHz
                            // bit 1-0: Clock determined by Config

    init(); // init IO, ADC, TMR2
    LED_POW = 0;

    delay_1sec();
    
    delay_50ms();
    while(!SW1);
    delay_50ms();

    TMR0CS = 0;
    TMR1 = 0;
    TMR1ON = 1;
    GIE = 1;

    freq = 0;
    freq_ref = 0;

    do{
        sum_freq = 0;
        freq_avg_old = freq_ref;
        for(i=0; i<8; i++){
            while(freq == 0);
            sum_freq += freq;
            freq = 0;
        }
        freq_ref = sum_freq >> 3;
        dlt_freq = freq_ref > freq_avg_old ? freq_ref - freq_avg_old: freq_avg_old - freq_ref;
    }while(freq_ref < FREQ_LOW || dlt_freq > DLT_STABLE);

    LED_POW = 1;
    
    while(1){
        freq = 0;
        while(freq == 0){
            if(IOCF_SW1)
                go_to_bed();
        }
        if(freq > freq_ref)
            dlt_freq = freq - freq_ref;
        else
            dlt_freq = freq_ref - freq;

        if(dlt_freq < DET_TH){
            TMR2ON = 0;
            LAT_LED = MUSK_LED_POW;
            //PR2 = 0xFF;
            //T2CON |= 0x03;
        }else{

            if(dlt_freq > DET_MAX){
                dlt_freq = DET_MAX;
            }
            dlt_freq >>= 1;
            if(dlt_freq <= 16){
                PR2 = 0xFF;
                T2CON |= 0x03;
                LAT_LED = LED_DET_1 | MUSK_LED_POW;
            }else if(dlt_freq <= 64){
                T2CON |= 0x03;  // PS = 1:64
                dlt_freq = 0x1000 / dlt_freq;
                PR2 = (char)dlt_freq;
                LAT_LED = LED_DET_1 | MUSK_LED_POW;
            }else{
                if(dlt_freq < 128)
                    LAT_LED = LED_DET_2 | MUSK_LED_POW;
                else
                    LAT_LED = LED_DET_3 | MUSK_LED_POW;
                T2CON &= 0xFE;  // PS = 1:16
                dlt_freq = 0x4000 / dlt_freq;
                PR2 = (char)dlt_freq;
            }
            CCPR1L = PR2 >> 1;
            TMR2ON = 1;
       }

 

    }

    return 0;
}

void interrupt isr(void){
    if(TMR1GIF){    // 65.5msec
        TMR1GIF = 0;
        freq = TMR1;
        TMR1 = 0;
        TMR0 = 0xFF;

        /*
        sum_freq += freq;
        i_avg++;
        if(i_avg > 8){
            sum_freq >>= 2;
            freq = sum_freq;
            i_avg = 0;
            sum_freq = 0;
            set7seg_hex(freq);
        }*/
        //flg = 1;
    }
    if(IOCIF){
        IOCIF = 0;
        IOCIE = 0;
        GIE = 0;
    }

}	

void go_to_bed(){
//    TEMP_SENS_VDD = 0;
    TMR0CS = 1; // TMR0 OFF
    LAT_LED = 0;
    TRISA = 0xFF;
    CCP1CON = 0;
    TMR2ON = 0;
    SPK_OFF();
    DACEN = 0;  // DAC OFF
    C1ON = 0;   // Comparator1 OFF

    delay_50ms();
    while(!SW1);
    delay_50ms();

    GIE = 0;
    IOCAF = 0;
    IOCIF = 0;
    IOCIE = 1;

    SLEEP();
    NOP();
    
    delay_50ms();
    while(!SW1);
    delay_50ms();

    asm("RESET");
}

